@@ -14,7 +14,6 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 import asyncio
-import concurrent
 import logging
 from types import TracebackType
 from google.cloud.sql.connector.instance import (
@@ -24,6 +23,7 @@ from google.cloud.sql.connector.instance import (
 import google.cloud.sql.connector.pymysql as pymysql
 import google.cloud.sql.connector.pg8000 as pg8000
 import google.cloud.sql.connector.pytds as pytds
+import google.cloud.sql.connector.asyncpg as asyncpg
 from google.cloud.sql.connector.utils import generate_keys
 from google.auth.credentials import Credentials
 from threading import Thread
@@ -32,6 +32,18 @@ from functools import partial
 
 logger = logging.getLogger(name=__name__)
 
+ASYNC_DRIVERS = ["asyncpg"]
+
+
+class ConnectorLoopError(Exception):
+    """
+    Raised when Connector.connect is called with Connector._loop
+        in an invalid state (event loop in current thread).
+    """
+
+    def __init__(self, *args: Any) -> None:
+        super(ConnectorLoopError, self).__init__(self, *args)
+
 
 class Connector:
     """A class to configure and create connections to Cloud SQL instances.
@@ -53,6 +65,11 @@ class Connector:
     :param credentials
         Credentials object used to authenticate connections to Cloud SQL server.
         If not specified, Application Default Credentials are used.
+
+    :type loop: asyncio.AbstractEventLoop
+    :param loop
+        Event loop to run asyncio tasks, if not specified, defaults to
+        creating new event loop on background thread.
     """
 
     def __init__(
@@ -61,13 +78,22 @@ class Connector:
         enable_iam_auth: bool = False,
         timeout: int = 30,
         credentials: Optional[Credentials] = None,
+        loop: asyncio.AbstractEventLoop = None,
     ) -> None:
-        self._loop: asyncio.AbstractEventLoop = asyncio.new_event_loop()
-        self._thread: Thread = Thread(target=self._loop.run_forever, daemon=True)
-        self._thread.start()
-        self._keys: concurrent.futures.Future = asyncio.run_coroutine_threadsafe(
-            generate_keys(), self._loop
-        )
+        # if event loop is given, use for background tasks
+        if loop:
+            self._loop: asyncio.AbstractEventLoop = loop
+            self._thread: Optional[Thread] = None
+            self._keys: asyncio.Future = loop.create_task(generate_keys())
+        # if no event loop is given, spin up new loop in background thread
+        else:
+            self._loop = asyncio.new_event_loop()
+            self._thread = Thread(target=self._loop.run_forever, daemon=True)
+            self._thread.start()
+            self._keys = asyncio.wrap_future(
+                asyncio.run_coroutine_threadsafe(generate_keys(), self._loop),
+                loop=self._loop,
+            )
         self._instances: Dict[str, Instance] = {}
 
         # set default params for connections
@@ -102,6 +128,18 @@ class Connector:
         :returns:
             A DB-API connection to the specified Cloud SQL instance.
         """
+        try:
+            # check if event loop is running in current thread
+            if self._loop == asyncio.get_running_loop():
+                raise ConnectorLoopError(
+                    "Connector event loop is running in current thread!"
+                    "Event loop must be attached to a different thread to prevent blocking code!"
+                )
+        # asyncio.get_running_loop will throw RunTimeError if no running loop is present
+        except RuntimeError:
+            pass
+
+        # if event loop is not in current thread, proceed with connection
         connect_task = asyncio.run_coroutine_threadsafe(
             self.connect_async(instance_connection_string, driver, **kwargs), self._loop
         )
@@ -123,7 +161,7 @@ class Connector:
         :type driver: str
         :param: driver:
             A string representing the driver to connect with. Supported drivers are
-            pymysql, pg8000, and pytds.
+            pymysql, pg8000, asyncpg, and pytds.
 
         :param kwargs:
             Pass in any driver-specific arguments needed to connect to the Cloud
@@ -133,7 +171,6 @@ class Connector:
         :returns:
             A DB-API connection to the specified Cloud SQL instance.
         """
-
         # Create an Instance object from the connection string.
         # The Instance should verify arguments.
         #
@@ -164,6 +201,7 @@ class Connector:
         connect_func = {
             "pymysql": pymysql.connect,
             "pg8000": pg8000.connect,
+            "asyncpg": asyncpg.connect,
             "pytds": pytds.connect,
         }
 
@@ -194,6 +232,10 @@ class Connector:
         # helper function to wrap in timeout
         async def get_connection() -> Any:
             instance_data, ip_address = await instance.connect_info(ip_type)
+            # async drivers are unblocking and can be awaited directly
+            if driver in ASYNC_DRIVERS:
+                return await connector(ip_address, instance_data.context, **kwargs)
+            # synchronous drivers are blocking and run using executor
             connect_partial = partial(
                 connector, ip_address, instance_data.context, **kwargs
             )
@@ -222,15 +264,76 @@ class Connector:
         """Exit context manager by closing Connector"""
         self.close()
 
+    async def __aenter__(self) -> Any:
+        """Enter async context manager by returning Connector object"""
+        return self
+
+    async def __aexit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> None:
+        """Exit async context manager by closing Connector"""
+        await self.close_async()
+
     def close(self) -> None:
         """Close Connector by stopping tasks and releasing resources."""
-        close_future = asyncio.run_coroutine_threadsafe(self._close(), loop=self._loop)
+        close_future = asyncio.run_coroutine_threadsafe(
+            self.close_async(), loop=self._loop
+        )
         # Will attempt to safely shut down tasks for 5s
         close_future.result(timeout=5)
+        # if background thread exists for Connector, clean it up
+        if self._thread:
+            # stop event loop running in background thread
+            self._loop.call_soon_threadsafe(self._loop.stop)
+            # wait for thread to finish closing (i.e. loop to stop)
+            self._thread.join()
 
-    async def _close(self) -> None:
+    async def close_async(self) -> None:
         """Helper function to cancel Instances' tasks
         and close aiohttp.ClientSession."""
         await asyncio.gather(
             *[instance.close() for instance in self._instances.values()]
         )
+
+
+async def create_async_connector(
+    ip_type: IPTypes = IPTypes.PUBLIC,
+    enable_iam_auth: bool = False,
+    timeout: int = 30,
+    credentials: Optional[Credentials] = None,
+    loop: asyncio.AbstractEventLoop = None,
+) -> Connector:
+    """
+    Create Connector object for asyncio connections that can auto-detect
+    and use current thread's running event loop.
+
+    :type ip_type: IPTypes
+    :param ip_type
+        The IP type (public or private)  used to connect. IP types
+        can be either IPTypes.PUBLIC or IPTypes.PRIVATE.
+
+    :type enable_iam_auth: bool
+    :param enable_iam_auth
+        Enables IAM based authentication (Postgres only).
+
+    :type timeout: int
+    :param timeout
+        The time limit for a connection before raising a TimeoutError.
+
+    :type credentials: google.auth.credentials.Credentials
+    :param credentials
+        Credentials object used to authenticate connections to Cloud SQL server.
+        If not specified, Application Default Credentials are used.
+
+    :type loop: asyncio.AbstractEventLoop
+    :param loop
+        Event loop to run asyncio tasks, if not specified, defaults
+        to current thread's running event loop.
+    """
+    # if no loop given, automatically detect running event loop
+    if loop is None:
+        loop = asyncio.get_running_loop()
+    return Connector(ip_type, enable_iam_auth, timeout, credentials, loop)
